/**
 * Class Ackermann provides an iterative computation of the Ackermann function
 * A(n,m)
 * 
 * @author bcyrill
 */

class Ackermann {

	// Create a stack structure for maximal 100 elements
	// (compare Stack.java)
	Stack stack = new Stack();

	/**
	 * Iterative Computation of the Ackermann function A(n,m)
	 * 
	 * @param n
	 * @param m
	 * @return A(n,m)
	 */
	public int ackermann(int n, int m) {
		// Stack initialized with values n,m
		stack.push(n);
		stack.push(m);

		// Iterative Calculation
		while (stack.getSP() > 1) {
			//stack.print();
			m = stack.pop();
			n = stack.pop();

			if (m == 0) {
				stack.push(--n);
				stack.push(1);
			} else if (n == 0) {
				stack.push(++m);
			} else {
				stack.push(n - 1);
				stack.push(n);
				stack.push(--m);
			}
		}
		return m;

	} // end ackermann()

	public static void main(String args[]) {
		Ackermann a = new Ackermann();

		if (args.length != 2) {
			System.out.println("Please enter 2 integer numbers  ");
			System.out.println("Example: java Ackermann 2 3");
			System.exit(1);
		}
		int n = Integer.parseInt(args[0]);
		int m = Integer.parseInt(args[1]);

		System.out.println(a.ackermann(n, m));
	}

} // end class Ackermann

