/**
 * Class InfToPost <br>
 * 
 * The class InfToPost allows to convert infix expressions to postix
 * expressions. <br>
 * See also lecture script: "Postfixumwandlung mittels Stack" (page 83) <br>
 * 
 * @author Silvia Santini <br>
 *         Informatik II - Series 4 - SS07
 */

public class InfToPost {

	Stack stack = new Stack(); // Create a stack with 100 elements

	/**
	 * Prints out the content of the input-array in postfix notation
	 * 
	 * @param input
	 *            Infix Eingabe als Char Array
	 */
	public void convert(char[] input) {
		int i = 0;

		while (i < input.length) {

			// push operands on stack
			if (input[i] == '+' || input[i] == '*') {
				stack.push(input[i]);
			}

			// From http://java.sun.com/j2se/1.4.2/docs/api/:
			//
			// "The Character class wraps a value of the primitive type
			// char in an object. An object of type Character contains
			// a single field whose type is char. In addition, the
			// Character class provides several methods for determining
			// a character's category (lowercase letter, digit, etc.)
			// and for converting characters from uppercase to lowercase
			// and vice versa."
			//
			// isDigit( char c ) is a method of the Character class, which
			// for a character c ascertains whether c is a digit or not.
			// The same functionality may be obtained through the
			// expression: "(c >= '0' && c <= '9')"

			if (Character.isDigit(input[i])) {
				System.out.print(" " + input[i]);
				while (Character.isDigit(input[i + 1])) {
					i++;
					System.out.print(input[i]);
				}
			}

			if (input[i] == ')') {
				System.out.print(" " + (char) stack.pop());
			}

			i++;
		} // end while()

		System.out.println();

		// Es befinden sich noch weitere Elemente auf dem Stack, die Eingabe war
		// nicht korrekt z.B. nicht vollstndig geklammert.
		if (!stack.empty()) {
			System.out.println("Error: uncorrect expression");
		}
	}

	// Test inputs:
	// ((13*26)+80)
	// (47+((5*((76+42)+19)))+3)

	public static void main(String[] args) {

		if (args.length == 0 || args.length > 1) {
			System.out.print("Invalid Input!");
			System.out.println(" Please enter an infix expression"
					+ " as one-string argument.");
			System.out.println("Example: java InfToPost \"((7+5)*9)\"");
			System.exit(1);
		}

		// convert input string in a char array
		char[] input = args[0].toCharArray();
		InfToPost expression = new InfToPost();

		// Convert and print coverted expression
		expression.convert(input);
	}
}
